@@ -399,6 +399,9 @@ function (object, head_dim, num_query_heads, num_key_value_heads,
 #' @param bias_constraint
 #' Constraint for dense layer kernels.
 #'
+#' @param seed
+#' Optional integer to seed the dropout layer.
+#'
 #' @param ...
 #' For forward/backward compatability.
 #'
@@ -418,7 +421,7 @@ function (inputs, num_heads, key_dim, value_dim = NULL, dropout = 0,
     use_bias = TRUE, output_shape = NULL, attention_axes = NULL,
     kernel_initializer = "glorot_uniform", bias_initializer = "zeros",
     kernel_regularizer = NULL, bias_regularizer = NULL, activity_regularizer = NULL,
-    kernel_constraint = NULL, bias_constraint = NULL, ...)
+    kernel_constraint = NULL, bias_constraint = NULL, seed = NULL, ...)
 {
     args <- capture_args(list(input_shape = normalize_shape,
         batch_size = as_integer, batch_input_shape = normalize_shape,
